import itertools
import warnings
import ast
from multiprocessing import Pool, cpu_count
from pathlib import Path

import arrow as ar
import pandas as pd
from tqdm import tqdm

warnings.filterwarnings("ignore")

from .cid import indices
from geocif import utils as ut
from geoprepare import base


def remove_duplicates(lst):
    """

    :param lst:
    :return:
    """
    return list(set([i for i in lst]))


def get_admin_zone(country, parser):
    """
    Get admin zone and admin column name from config file for the given country.
    Falls back to DEFAULT section if not specified for country.
    
    :param country: Country name
    :param parser: ConfigParser object
    :return: tuple of (admin_zone, admin_col_name)
    """
    country = country.title().replace(" ", "_")
    country_lower = country.lower()
    
    # Try to get admin_zone from country-specific section
    if parser.has_section(country_lower) and parser.has_option(country_lower, "admin_zone"):
        admin_zone = parser.get(country_lower, "admin_zone")
    # Fall back to DEFAULT section
    elif parser.has_option("DEFAULT", "admin_zone"):
        admin_zone = parser.get("DEFAULT", "admin_zone")
    else:
        raise ValueError(f"admin_zone not specified for country {country} in config file.")
    
    # Try to get admin_col_name from country-specific section
    if parser.has_section(country_lower) and parser.has_option(country_lower, "admin_col_name"):
        admin_col_name = parser.get(country_lower, "admin_col_name")
    # Fall back to DEFAULT section
    elif parser.has_option("DEFAULT", "admin_col_name"):
        admin_col_name = parser.get("DEFAULT", "admin_col_name")
    # Final fallback
    else:
        raise ValueError(f"admin_col_name not specified for country {country} in config file.")
    
    return admin_zone, admin_col_name


def get_crops(country, parser):
    """
    Get crops list from config file for the given country.
    Falls back to DEFAULT section if not specified for country.
    
    :param country: Country name
    :param parser: ConfigParser object
    :return: list of crops
    """
    country_lower = country.lower().replace(" ", "_")
    
    # Try to get crops from country-specific section
    if parser.has_section(country_lower) and parser.has_option(country_lower, "crops"):
        crops_str = parser.get(country_lower, "crops")
    # Fall back to DEFAULT section
    elif parser.has_option("DEFAULT", "crops"):
        crops_str = parser.get("DEFAULT", "crops")
    else:
        raise ValueError(f"crops not specified for country {country} in config file.")
    
    return ast.literal_eval(crops_str)


def get_seasons(country, parser):
    """
    Get seasons list from config file for the given country.
    Falls back to DEFAULT section if not specified for country.
    Default is [1] if not specified anywhere.
    
    :param country: Country name
    :param parser: ConfigParser object
    :return: list of seasons (integers)
    """
    country_lower = country.lower().replace(" ", "_")
    
    # Try to get seasons from country-specific section
    if parser.has_section(country_lower) and parser.has_option(country_lower, "seasons"):
        seasons_str = parser.get(country_lower, "seasons")
        return ast.literal_eval(seasons_str)
    # Fall back to DEFAULT section
    elif parser.has_option("DEFAULT", "seasons"):
        seasons_str = parser.get("DEFAULT", "seasons")
        return ast.literal_eval(seasons_str)
    # Final fallback - default to season 1
    else:
        return [1]


def get_input_file_path(country, parser, data_source="harvest"):
    """
    Get input file path from config file for the given country.
    Path depends on data_source flag:
        - 'harvest': ${PATHS:dir_output}/{project_name}/crop_t{floor}/{country}/
        - 'agmet': ${PATHS:dir_crop_inputs}/processed/{country}/
    
    :param country: Country name
    :param parser: ConfigParser object
    :param data_source: 'harvest' or 'agmet'
    :return: Path object for input files
    """
    country_lower = country.lower().replace(" ", "_")
    
    if data_source == "harvest":
        # Use dir_output/{project_name}/{dir_threshold}/{country}/ path
        # project_name and dir_threshold come from geoextract.txt config
        dir_output = parser.get("PATHS", "dir_output")
        project_name = parser.get("PROJECT", "project_name")
        if parser.has_option("DEFAULT", "threshold") and parser.getboolean("DEFAULT", "threshold"):
            floor = parser.getint("DEFAULT", "floor")
            path_str = f"{dir_output}/{project_name}/crop_t{floor}/{country_lower}"
        else:
            path_str = f"{dir_output}/{project_name}/{country_lower}"
    else:
        # Use agmet path - check country-specific first, then DEFAULT
        if parser.has_section(country_lower) and parser.has_option(country_lower, "input_file_path"):
            path_str = parser.get(country_lower, "input_file_path")
        elif parser.has_option("DEFAULT", "input_file_path"):
            path_str = parser.get("DEFAULT", "input_file_path")
        else:
            raise ValueError(f"input_file_path not specified for country {country} in config file.")
        
        # Append country subdirectory for agmet
        path_str = f"{path_str}/{country_lower}"
    
    return Path(path_str)


class cei_runner(base.BaseGeo):
    def __init__(self, path_config_file):
        super().__init__(path_config_file)

        # Parse configuration files
        self.parse_config()

        # Get data_source from config (default: 'harvest')
        if self.parser.has_option("DEFAULT", "data_source"):
            self.data_source = self.parser.get("DEFAULT", "data_source").lower()
        else:
            self.data_source = "harvest"
        
        # Validate data_source
        if self.data_source not in ["harvest", "agmet"]:
            raise ValueError(f"Invalid data_source: {self.data_source}. Must be 'harvest' or 'agmet'.")

        # Get base input path from DEFAULT section (for agmet mode)
        if self.parser.has_option("DEFAULT", "input_file_path"):
            self.base_dir = Path(self.parser.get("DEFAULT", "input_file_path"))
        elif self.data_source == "agmet":
            raise ValueError("input_file_path not specified in DEFAULT section of config file (required for agmet mode).")
        
        self.do_parallel = self.parser.getboolean("DEFAULT", "do_parallel")
        
        # Read countries and methods from config file
        self.countries = ast.literal_eval(self.parser.get("DEFAULT", "countries"))
        self.method = self.parser.get("DEFAULT", "method")

    def collect_files_harvest(self):
        """
        Collect files for 'harvest' data source.
        Reads specific files with pattern: {country}_{crop}_s{season}.csv
        from ${PATHS:dir_output}/{project_name}/crop_t{floor}/{country}/
        
        :return: DataFrame with file information
        """
        df_files = pd.DataFrame(columns=["directory", "path", "filename", "admin_zone", "admin_col_name"])
        
        for country in self.countries:
            country_lower = country.lower().replace(" ", "_")
            country_path = get_input_file_path(country, self.parser, data_source="harvest")
            
            # Get crops and seasons for this country
            crops = get_crops(country, self.parser)
            seasons = get_seasons(country, self.parser)
            
            # Get admin_zone and admin_col_name from config file
            admin_zone, admin_col_name = get_admin_zone(country, self.parser)
            
            for crop in crops:
                for season in seasons:
                    # Construct expected filename
                    filename = f"{country_lower}_{crop}_s{season}.csv"
                    filepath = country_path / filename
                    
                    if filepath.exists():
                        # For harvest mode, directory is 'countries'
                        process_type = "countries"
                        
                        # Add to dataframe
                        df_files.loc[len(df_files)] = [
                            process_type, 
                            filepath, 
                            filename, 
                            admin_zone, 
                            admin_col_name
                        ]
                    else:
                        print(f"Warning: Expected file not found: {filepath}")
        
        return df_files

    def collect_files_agmet(self):
        """
        Collect files for 'agmet' data source.
        Recursively finds all CSV files in the processed directory.
        
        :return: DataFrame with file information
        """
        df_files = pd.DataFrame(columns=["directory", "path", "filename", "admin_zone", "admin_col_name"])
        
        # If specific countries are defined, check their paths individually
        if self.countries and self.countries != ['all']:
            for country in self.countries:
                country_path = get_input_file_path(country, self.parser, data_source="agmet")
                
                # Go up one level since get_input_file_path now includes country
                # and we want to search from the country directory
                for filepath in country_path.rglob("*.csv"):
                    country_name = filepath.parents[0].name

                    # Get admin_zone and admin_col_name from config file
                    admin_zone, admin_col_name = get_admin_zone(country_name, self.parser)

                    # HACK: Skip korea for now, as it is giving errors
                    if country_name == "republic_of_korea":
                        continue

                    # Get name of directory one level up
                    process_type = filepath.parents[1].name

                    # Get name of file
                    filename = filepath.name

                    # Add to dataframe
                    df_files.loc[len(df_files)] = [process_type, filepath, filename, admin_zone, admin_col_name]
        else:
            # Use base directory for all countries
            for filepath in self.base_dir.rglob("*.csv"):
                country = filepath.parents[0].name

                # Get admin_zone and admin_col_name from config file
                admin_zone, admin_col_name = get_admin_zone(country, self.parser)

                # HACK: Skip korea for now, as it is giving errors
                if country == "republic_of_korea":
                    continue

                # Get name of directory one level up
                process_type = filepath.parents[1].name

                # Get name of file
                filename = filepath.name

                # Add to dataframe
                df_files.loc[len(df_files)] = [process_type, filepath, filename, admin_zone, admin_col_name]

        # Exclude those rows where directory is processed and file is already in
        # processed_include_fall directory
        no_fall = df_files["directory"] == "processed"
        include_fall = df_files[df_files["directory"] == "processed_include_fall"][
            "filename"
        ]

        df_files = df_files[~(no_fall & (df_files["filename"].isin(include_fall)))]

        return df_files

    def collect_files(self):
        """
        Collect files based on data_source configuration.
        
        1. If data_source='harvest': Read specific files {country}_{crop}_s{season}.csv
           from ${PATHS:dir_output}/{project_name}/crop_t{floor}/{country}/
        2. If data_source='agmet': Recursively find all CSVs in processed directory
        
        :return: DataFrame with columns: directory, path, filename, admin_zone, admin_col_name
        """
        if self.data_source == "harvest":
            return self.collect_files_harvest()
        else:
            return self.collect_files_agmet()

    def process_combinations(self, df, method):
        """
        Create a list of tuples of the following:
            - directory: name of directory where file is located
            - path: full path to file
            - filename: name of file
            - method: whether to compute indices for phenological stages or not
        This tuple will be used as input to the `process` function
        :param df:
        :param method:
        :return:
        """
        combinations = []

        for index, row in tqdm(df.iterrows()):
            combinations.extend(
                list(
                    itertools.product([row["directory"]], [row["path"]], [row["filename"]], [row["admin_zone"]], [method])
                )
            )

        combinations = remove_duplicates(combinations)

        return combinations

    def main(self):
        """

        :param method:
        :return:
        """
        # Create a dataframe of the files to be analyzed
        df_files = self.collect_files()
        
        if df_files.empty:
            print(f"No files found for data_source='{self.data_source}' and countries={self.countries}")
            return

        # Extract unique crops from filenames for display
        crops_found = sorted(set(
            row["filename"].rsplit("_s", 1)[0].split("_", 1)[-1]
            for _, row in df_files.iterrows()
            if "_s" in row["filename"]
        ))

        params = [
            ("Countries", self.countries),
            ("Crops", crops_found if crops_found else ["(from filenames)"]),
            ("Data source", self.data_source),
            ("Method", self.method),
            ("Parallel", str(self.do_parallel)),
            ("Files to process", str(len(df_files))),
            ("Year range", f"2001-{ar.utcnow().year}"),
        ]
        ut.display_run_summary("CEI Indices Runner", params, wait=20)

        combinations = self.process_combinations(df_files, self.method)

        # Add an element to the tuple to indicate the season
        # Last element is redo flag which is True if the analysis is to be redone
        # and False otherwise. Analysis is always redone for the current year
        # and last year whether file exists or not
        combinations = [
            (
                self.parser,
                status,
                path,
                filename,
                admin_zone,
                category,
                year,
                "ndvi",
                False,  # redo
            )
            for year in range(2001, ar.utcnow().year + 1)
            for status, path, filename, admin_zone, category in combinations
        ]

        # Filter combinations based on countries from config file
        if self.countries and self.countries != ['all']:
            combinations = [
                i
                for i in combinations
                if any(country.lower().replace(" ", "_") in i[3].lower() 
                       for country in self.countries)
            ]

        if self.do_parallel:
            num_cpu = int(cpu_count() * 0.75)
            with Pool(num_cpu) as p:
                for i, _ in enumerate(p.imap_unordered(indices.process, combinations)):
                    pass
        else:
            # Use the code below if you want to test without parallelization or
            # if you want to debug by using pdb
            pbar = tqdm(combinations)
            for i, val in enumerate(pbar):
                pbar.set_description(
                    f"Main loop {combinations[i][2]} {combinations[i][5]}"
                )
                indices.process(val)


def run(path_config_files=[]):
    """

    Args:
        path_config_files:

    Returns:

    """
    """ Check dictionary keys to have no spaces"""
    indices.validate_index_definitions()

    obj = cei_runner(path_config_files)
    
    # Iterate over methods from config file
    obj.main()


if __name__ == "__main__":
    run()


import geopandas as gpd
import pygmt
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
import matplotlib.patches as mpatches
import os
filtered_shapefile_path = r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Regions\Shps\filtered_shapefile5.shp"

if not os.path.isfile(filtered_shapefile_path):

    # Load the shapefile using GeoPandas
    shapefile_path = r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Regions\Shps\adm_shapefile.shp"
    gdf = gpd.read_file(shapefile_path, engine="pyogrio")

    # Only keep one row per ADMIN0
    gdf = gdf.drop_duplicates(subset="ADMIN0")

    sh2_path = r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Regions\Shps\Level_1.shp"
    gdf2 = gpd.read_file(sh2_path, engine="pyogrio")

    # Subset gdf2 to USA, Pakistan and Afghanistan
    gdf2 = gdf2[gdf2["ADM0_NAME"].isin(["United States of America"])]

    # Exclude Alska and Hawaii from the USA
    gdf2 = gdf2[~gdf2["ADM1_NAME"].isin(["Alaska", "Hawaii"])]

    # Now combine all the states into one polygon
    gdf2 = gdf2.dissolve(by="ADM0_NAME")
    gdf2 = gdf2.reset_index()

    # Rename ADM0_NAME to ADMIN0 for consistency
    gdf2.rename(columns={"ADM0_NAME": "ADMIN0"}, inplace=True)

    # Only keep ADMIN0 and geometry columns in gdf and gdf2
    gdf = gdf[["ADMIN0", "geometry"]]
    gdf2 = gdf2[["ADMIN0", "geometry"]]

    # Merge gdf and gdf2
    import pandas as pd
    gdf = pd.concat([gdf, gdf2], ignore_index=True)

    # Save the filtered shapefile as a temporary file

    gdf.to_file(filtered_shapefile_path)
else:
    gdf = gpd.read_file(filtered_shapefile_path, engine="pyogrio")

# Create the global map with highlighted countries
fig = pygmt.Figure()

# Define the region of interest and projection
# fig.basemap(region="g", projection="R12c/20", frame=True)
fig.basemap(region=[-135, 60, -35, 53], projection="Q12c", frame=True)

# Use the coast function to draw land and water
fig.coast(land="lightgray", water="lightcyan")

# Highlight the countries using the filtered shapefile
fig.plot(data=filtered_shapefile_path, pen="0.35p,black")

# Add hatches to Pakistan and Afghanistan
gdf_filled = gdf[gdf["ADMIN0"].isin(["Pakistan", "Afghanistan"])]
for _, row in gdf_filled.iterrows():
    fill_gdf = gpd.GeoDataFrame([row], columns=gdf.columns)
    with pygmt.helpers.GMTTempFile() as tmpfile:
        fill_gdf.to_file(tmpfile.name, driver="GeoJSON")
        fig.plot(data=tmpfile.name, pen="0.35p,black", fill="black@50+h")

# Save the figure
fig.savefig("global_choropleth_highlighted_v1.png", dpi=1000)

# Show the figure
fig.show()

import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cartopy.feature as cfeature

from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER

import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cartopy.feature as cfeature
def create_map_with_ticks():
    fig, ax = plt.subplots(figsize=(10, 5), subplot_kw={'projection': ccrs.PlateCarree()})

    # Add features to the map
    ax.add_feature(cfeature.LAND, edgecolor='black')
    ax.add_feature(cfeature.COASTLINE)

    # Set the extent (min_lon, max_lon, min_lat, max_lat)
    ax.set_extent([-120, -70, 25, 50], crs=ccrs.PlateCarree())

    # Customize ticks to mimic GMT style
    from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
    import matplotlib.ticker as mticker

    ax.set_xticks([-120, -110, -100, -90, -80, -70], crs=ccrs.PlateCarree())
    ax.set_yticks([25, 30, 35, 40, 45, 50], crs=ccrs.PlateCarree())

    lon_formatter = LongitudeFormatter(number_format='g', degree_symbol='', dateline_direction_label=True)
    lat_formatter = LatitudeFormatter(number_format='g', degree_symbol='')

    ax.xaxis.set_major_formatter(lon_formatter)
    ax.yaxis.set_major_formatter(lat_formatter)

    # Adding gridlines
    gl = ax.gridlines(crs=ccrs.PlateCarree(), draw_labels=False, linewidth=1, color='gray', alpha=0.5, linestyle='--')
    gl.xlocator = mticker.FixedLocator([-120, -110, -100, -90, -80, -70])
    gl.ylocator = mticker.FixedLocator([25, 30, 35, 40, 45, 50])

    # Display the map
    plt.show()


create_map_with_ticks()

breakpoint()
from matplotlib.cm import ScalarMappable
from matplotlib.colors import ListedColormap, Normalize
import matplotlib.pyplot as plt
import matplotlib as mpl

import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap, Normalize
from matplotlib.cm import ScalarMappable

fig, ax = plt.subplots(figsize=(3, 0.5))

colors = ["red", "cyan", "darkgreen"]
cmap = ListedColormap(colors)
norm = Normalize(vmin=2.5, vmax=5.5)
cb = fig.colorbar(
    mappable=ScalarMappable(norm=norm, cmap=cmap),
    cax=ax,
    ticks=[3.5, 4.5],
    ticklocation="top",
    orientation="horizontal",
    drawedges=True,
)
cb.solids.set(edgecolor="white", linewidth=5)
cb.outline.set_visible(False)
cb.dividers.set(linewidth=1, alpha=0.6)
cb.ax.tick_params(width=1, length=10, color="k")
plt.setp(cb.ax.xaxis.get_ticklines(), alpha=0.6)
cb.set_ticklabels([3.5, 4.5], alpha=0.6, color="k", fontsize=15, fontfamily="Arial")
plt.savefig(r"D:\Users\ritvik\projects\GEOGLAM\Output\fao\output\ml\db\aa.png")
breakpoint()
fig, ax = plt.subplots(figsize=(6, 1), layout="constrained")
cmap = ListedColormap(["red", "cyan", "slategrey"])
norm = mpl.colors.Normalize(vmin=5, vmax=10)
sm = ScalarMappable(norm=Normalize(2.5, 5.5), cmap=cmap)
fig.colorbar(sm, cmap=cmap, ticks=[3.5, 4.5], cax=ax, orientation="horizontal", label="Colorbar")
plt.show()
cb = plt.colorbar(
    sm, cmap=cmap, ticks=[3.5, 4.5], cax=ax, orientation="horizontal", label="Colorbar"
)

# Add number on top of the colorbar

breakpoint()
df = pd.read_csv(
    r"D:\Users\ritvik\projects\GEOGLAM\Output\fao\output\ml\analysis\April-16-2024\aa.csv"
)
# Initialize the Matplotlib figure and axis
fig, ax = plt.subplots(figsize=(14, 10))

# Plot the predicted yield as a dot and the CI as a vertical line for each region
for i in range(len(df)):
    ax.plot([i, i], [df["Lower CI"][i], df["Upper CI"][i]], color="gray")  # CI line
    ax.plot(i, df["Predicted"][i], "ko")  # Predicted yield dot
    ax.plot(i, df["Median"][i], "ro")  # Median yield differently colored dot

# Add labels and title
ax.set_xticks(range(len(df)))
ax.set_xticklabels(df["Region"], rotation=90)
ax.set_ylabel("Yield (tn per ha)")
ax.set_title("Predicted and Median Yields with Confidence Intervals per Region")

# Show the plot with a tight layout
plt.tight_layout()
plt.show()
breakpoint()

from osgeo import ogr

driver = ogr.GetDriverByName("FileGDB")

ds = driver.Open(r"C:\Users\ritvik\Downloads\Admin_1_Regions.gdb", 0)

breakpoint()


import numpy as np

import geopandas as gpd

dg_ewcm_geoglam = gpd.read_file(
    r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Regions\Shps\EWCM_Level_1.shp",
    engine="pyogrio",
)

dg_ewcm_fewsnet = gpd.read_file(
    r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Regions\Shps\hvstat_shape (1).gpkg",
    engine="pyogrio",
)

# Only select thiose rows in dg_ewcm_fewsnet where ADMIN2 is not None
dg_ewcm_fewsnet = dg_ewcm_fewsnet[dg_ewcm_fewsnet["ADMIN2"].notnull()]

# Find countries in dg_ewcm_fewsnet
countries = dg_ewcm_fewsnet[dg_ewcm_fewsnet["ADMIN0"].notnull()]["ADMIN0"].unique()

# Restrict dg_ewcm_geoglam to countries in dg_ewcm_fewsnet
dg_ewcm_geoglam = dg_ewcm_geoglam[dg_ewcm_geoglam["ADM0_NAME"].isin(countries)]
# Find polygons that exist in dg_ewcm_geoglam but not in dg_ewcm_fewsnet
# print("GEOGLAM but not in FEWSNET", dg_ewcm_geoglam[~dg_ewcm_geoglam.geometry.isin(dg_ewcm_fewsnet.geometry)])
a1 = dg_ewcm_geoglam[~dg_ewcm_geoglam.geometry.isin(dg_ewcm_fewsnet.geometry)]
# Find polygons that exist in dg_ewcm_fewsnet but not in dg_ewcm_geoglam
# print("FEWSNET but not in GEOGLAM", dg_ewcm_fewsnet[~dg_ewcm_fewsnet.geometry.isin(dg_ewcm_geoglam.geometry)])
a2 = dg_ewcm_fewsnet[~dg_ewcm_fewsnet.geometry.isin(dg_ewcm_geoglam.geometry)]

# Remove geometry from a1 and a2
a1 = a1.drop(columns=["geometry"])
a2 = a2.drop(columns=["geometry"])

# Output to disk
a1.to_csv(r"D:\Users\ritvik\projects\GEOGLAM\in_geoglam_but_not_in_fewsnet.csv", index=False)
a2.to_csv(r"D:\Users\ritvik\projects\GEOGLAM\in_fewsnet_but_not_in_geoglam.csv", index=False)
breakpoint()


def compute_h_index(values):
    # Sort the citations array in descending order
    sorted_value = np.sort(values)[::-1]
    # Iterate through the sorted citations to find the h-index
    h_index = 0
    for i, value in enumerate(sorted_value, start=1):
        if value >= i:
            h_index = value
        else:
            break

    return h_index


# Example usage
citations = np.array([3.1, 0.56, 10.02, 2.35, 5.60])
h_index = compute_h_index(citations)
print(f"The h-index is: {h_index}")

breakpoint()
import numpy as np
import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
import shap
import matplotlib.pyplot as plt

# Generate synthetic data
np.random.seed(0)
X = np.random.normal(size=(100, 4))
y = np.random.binomial(1, 0.5, size=100)

# Split the data
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# Train a simple RandomForest model
model = RandomForestClassifier(n_estimators=10, max_depth=3, random_state=42)
model.fit(X_train, y_train)

# Compute SHAP values
explainer = shap.TreeExplainer(model)
shap_values = explainer.shap_values(X_test)

# Plot with default font sizes
# Beeswarm plot
shap.summary_plot(shap_values[1], X_test, plot_type="dot")

# Waterfall plot for a single prediction
sample_idx = 0  # Example index for demonstration
shap.plots.waterfall(shap_values[1][sample_idx], max_display=10)

# Adjust font sizes globally for subsequent plots
plt.rcParams.update({"font.size": 8})  # Adjust the font size as needed

# Beeswarm plot with adjusted font size
shap.summary_plot(shap_values[1], X_test, plot_type="dot")

# Waterfall plot with adjusted font size for a single prediction
shap.plots.waterfall(shap_values[1][sample_idx], max_display=10)

# Reset matplotlib font settings to default if needed
plt.rcParams.update(plt.rcParamsDefault)

breakpoint()


def plot_dekads():
    # Correcting the generation of dekad start dates
    start_date = pd.Timestamp(year=2021, month=1, day=1)
    dekad_starts = []

    for month in range(1, 13):  # Loop through each month
        for day in [1, 11, 21]:
            dekad_date = pd.Timestamp(year=2021, month=month, day=day)
            dekad_starts.append(dekad_date)

    # Remove the extra dekad start if it goes beyond the year
    if dekad_starts[-1] > pd.Timestamp(year=2021, month=12, day=31):
        dekad_starts.pop()

    fig, ax = plt.subplots(figsize=(20, 3))

    color_nov_to_apr = "lightblue"  # Light blue for Nov 1 to Apr 30
    color_default = "lightgray"  # Light gray for the rest of the year

    for i, date in enumerate(dekad_starts):
        if date.month >= 11 or date.month <= 4:
            facecolor = color_nov_to_apr
        else:
            facecolor = color_default

        # Adding squares with specified face color and white border
        ax.add_patch(
            patches.Rectangle((i, 0.5), 1, 1, edgecolor="white", facecolor=facecolor, linewidth=2)
        )

        # Non-rotated annotations, slightly increased size
        annotation = f"D{i + 1}"
        ax.text(
            i + 0.5,
            0.35,
            annotation,
            ha="center",
            va="top",
            fontsize=10,
            fontweight="bold",
        )

    ax.set_xlim(0, len(dekad_starts))
    ax.set_ylim(0, 1.75)
    ax.set_aspect("equal")
    ax.axis("off")
    ax.set_facecolor("whitesmoke")
    # plt.title('Dekad Start Dates with White Borders', fontsize=12, fontweight='bold')
    plt.tight_layout()
    plt.show()


def plot_dekads_with_arrows():
    # Correctly generating dekad start dates
    start_date = pd.Timestamp(year=2021, month=1, day=1)
    dekad_starts = []

    for month in range(1, 13):  # Loop through each month
        for day in [1, 11, 21]:
            dekad_date = pd.Timestamp(year=2021, month=month, day=day)
            dekad_starts.append(dekad_date)

    # Adjust if the last date exceeds the year
    if dekad_starts[-1] > pd.Timestamp(year=2021, month=12, day=31):
        dekad_starts.pop()

    # Plotting with annotations, colored squares, and arrows
    fig, ax = plt.subplots(figsize=(20, 4))  # Increased figure height to accommodate arrows

    color_nov_to_apr = "lightblue"  # Light blue for Nov 1 to Apr 30
    color_default = "lightgray"  # Light gray for the rest of the year

    for i, date in enumerate(dekad_starts):
        facecolor = color_nov_to_apr if date.month >= 11 or date.month <= 4 else color_default
        # Draw the rectangles with specified face color and white border
        ax.add_patch(
            patches.Rectangle((i, 0.5), 1, 1, edgecolor="white", facecolor=facecolor, linewidth=2)
        )
        # Add annotation
        ax.text(
            i + 0.5,
            0.35,
            f"D{i + 1}",
            ha="center",
            va="top",
            fontsize=10,
            fontweight="bold",
        )

    # Drawing arrows from center to center, maintaining vertical positions
    # The vertical positions remain the same as before (-0.2, -0.6, -1.0) for visual consistency
    arrow_starts_ends = [
        (7.5, -0.2, 6.5, -0.2),
        (7.5, -0.6, 5.5, -0.6),
        (7.5, -1.0, 4.5, -1.0),
    ]

    for x_start, y_start, x_end, y_end in arrow_starts_ends:
        ax.annotate(
            "",
            xy=(x_end, y_end),
            xycoords="data",
            xytext=(x_start, y_start),
            textcoords="data",
            arrowprops=dict(arrowstyle="->", color="black", lw=1.5),
        )

    ax.set_xlim(0, len(dekad_starts))
    ax.set_ylim(-1.25, 1.75)  # Adjusted to accommodate arrows
    ax.set_aspect("equal")
    ax.axis("off")
    ax.set_facecolor("whitesmoke")
    # plt.title('Dekad Start Dates with Centered Arrows', fontsize=12, fontweight='bold')
    plt.tight_layout()
    plt.show()


def plot_dekads_v2():
    # Correctly generating dekad start dates
    start_date = pd.Timestamp(year=2021, month=1, day=1)
    dekad_starts = []

    for month in range(1, 13):  # Loop through each month
        for day in [1, 11, 21]:
            dekad_date = pd.Timestamp(year=2021, month=month, day=day)
            dekad_starts.append(dekad_date)

    # Adjust if the last date exceeds the year
    if dekad_starts[-1] > pd.Timestamp(year=2021, month=12, day=31):
        dekad_starts.pop()

    # Plotting with increased space for annotations
    fig, ax = plt.subplots(
        figsize=(20, 4)
    )  # Adjusted figure size for clarity and additional spacing

    color_nov_to_apr = "lightblue"  # Light blue for Nov 1 to Apr 30
    color_default = "lightgray"  # Light gray for the rest of the year

    for i, date in enumerate(dekad_starts):
        facecolor = color_nov_to_apr if date.month >= 11 or date.month <= 4 else color_default
        # Draw the rectangles with specified face color and white border
        ax.add_patch(
            patches.Rectangle((i, 0.5), 1, 1, edgecolor="white", facecolor=facecolor, linewidth=2)
        )
        # Add "D" annotation
        ax.text(
            i + 0.5,
            0.35,
            f"D{i + 1}",
            ha="center",
            va="top",
            fontsize=10,
            fontweight="bold",
        )
        # Further increase space and then add date annotation
        ax.text(
            i + 0.5,
            0.1,
            date.strftime("%b %d"),
            ha="center",
            va="top",
            fontsize=8,
            fontstyle="italic",
        )

    ax.set_xlim(0, len(dekad_starts))
    ax.set_ylim(-0.5, 1.75)  # Adjusted to better accommodate the increased spacing
    ax.set_aspect("equal")
    ax.axis("off")
    ax.set_facecolor("whitesmoke")
    # plt.title('Dekad Start Dates with Increased Space for Annotations', fontsize=12, fontweight='bold')
    plt.tight_layout()
    plt.show()


if __name__ == "__main__":
    plot_dekads()
    plot_dekads_v2()
    plot_dekads_with_arrows()



import rasterio
from rasterio.warp import calculate_default_transform, reproject, Resampling
import numpy as np
import matplotlib.pyplot as plt
import math

# Input / Output paths
input_path = r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Masks\wolayita_maize.tif"
output_path = r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Masks\wolayita_maize_5km_percentage.tif"

import rasterio
from rasterio.warp import calculate_default_transform, reproject, Resampling
from math import ceil
import numpy as np

input_path = r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Masks\wolayita_maize.tif"
output_path = r"D:\Users\ritvik\projects\GEOGLAM\Input\Global_Datasets\Masks\wolayita_maize_5km_percentage.tif"

with rasterio.open(input_path) as src:
    # 1) If needed, assign correct CRS
    # Example: if you know it's actually EPSG:32637 but isn't set
    # src_crs = rasterio.crs.CRS.from_epsg(32637)
    # else if it's already correct, do:
    src_crs = src.crs

    # 2) Decide your pixel size.
    # If src_crs is lat/lon (EPSG:4326), use ~0.045 deg for ~5 km.
    # If src_crs is UTM in meters, use 5000 for 5 km.
    pixel_size = 0.045  # or 5000 if in meters

    transform, width, height = calculate_default_transform(
        src_crs,  # source crs
        src_crs,  # target crs (same if you just want coarser in place)
        src.width,
        src.height,
        *src.bounds,
        resolution=pixel_size
    )

    # Prepare output fraction array
    fraction_array = np.full((height, width), -9999, dtype=np.float32)

    # Reproject with average -> fraction
    reproject(
        source=rasterio.band(src, 1),
        destination=fraction_array,
        src_transform=src.transform,
        src_crs=src_crs,
        dst_transform=transform,
        dst_crs=src_crs,
        resampling=Resampling.average,
        dst_nodata=-9999
    )

# Now fraction_array should have values in [0..1], with -9999 for nodata.
valid_mask = (fraction_array != -9999)

if not np.any(valid_mask):
    print("No valid cells at all (everything is nodata). This indicates a bounding box or CRS mismatch.")
else:
    frac_min = fraction_array[valid_mask].min()
    frac_max = fraction_array[valid_mask].max()
    print("Fraction min:", frac_min)
    print("Fraction max:", frac_max)

    # If both min and max are 0.0, it means there's truly no coverage or it's extremely small.
    # Otherwise you might see something like 0.0, 0.01, 0.5, etc.

    # Then let's see if maybe they're all below 0.005:
    below_005 = (fraction_array[valid_mask] < 0.005).all()
    print("All fractions < 0.5%?", below_005)

breakpoint()
with rasterio.open(input_path) as src:
    # If src.crs is None but you KNOW it's EPSG:4326, assign it:
    # src_crs = rasterio.crs.CRS.from_epsg(4326)
    # Otherwise, just use what's in the file:
    src_crs = src.crs

    # Let's assume the file is already lat/lon (EPSG:4326).
    # We'll define ~0.045° as "5 km" at the equator.
    new_res = 0.045

    # Calculate a new transform and new shape
    # for coarser resolution in the SAME EPSG:4326.
    transform, width, height = calculate_default_transform(
        src_crs,  # src CRS
        src_crs,  # dst CRS (same if you want to stay in lat/lon)
        src.width,
        src.height,
        *src.bounds,
        resolution=new_res  # sets pixel size to 0.045 degrees
    )

    # Read full data for histogram plotting
    data_in = src.read(1, masked=True)
    in_profile = src.profile.copy()

# Plot input histogram (0 or 1)
arr_in = data_in.compressed()
plt.figure()
plt.hist(arr_in, bins=[-0.5, 0.5, 1.5], edgecolor='black')
plt.title("Input (0/1)")
plt.xlabel("Value")
plt.ylabel("Frequency")
plt.show()

# Prepare output array, float32 with sentinel -9999
out_array = np.full((height, width), -9999, dtype=np.float32)

with rasterio.open(input_path) as src:
    reproject(
        source=rasterio.band(src, 1),
        destination=out_array,
        src_transform=src.transform,
        src_crs=src.crs,
        dst_transform=transform,
        dst_crs=src_crs,  # same
        resampling=Resampling.average,
        dst_nodata=-9999
    )

# Now out_array has fraction in [0..1]. Convert to % (0..100).
breakpoint()
mask_valid = (out_array != -9999)
out_array[mask_valid] *= 100.0
out_array[mask_valid] = np.rint(out_array[mask_valid])  # round
out_array = out_array.astype(np.int32)

# Update profile
out_profile = in_profile.copy()
out_profile.update({
    'driver': 'GTiff',
    'width': width,
    'height': height,
    'transform': transform,
    'crs': src_crs,
    'dtype': 'int32',
    'nodata': -9999
})

# Write out
with rasterio.open(output_path, 'w', **out_profile) as dst:
    dst.write(out_array, 1)

print("Wrote:", output_path)

# Plot histogram of output (ignore -9999)
out_data = np.where(out_array == -9999, np.nan, out_array)
valid_data = out_data[~np.isnan(out_data)]
plt.figure()
plt.hist(valid_data, bins=50, edgecolor="black")
plt.title("5km Percentage (0-100)")
plt.xlabel("Percent cropped")
plt.ylabel("Frequency")
plt.show()



import ast
import itertools
import math

import arrow as ar
import numpy as np
import pandas as pd
from heapq import nsmallest
from pathlib import Path
from tqdm import tqdm

from geoprepare import base
from geocif.agmet import plot, utils


class AgmetGeo(base.BaseGeo):
    """Lightweight geo object for agmet plots. No Neptune dependency."""

    def __init__(self, path_config_file):
        super().__init__(path_config_file)
        self.parse_config()

    def _get_option(self, option, default=None, sections=("AGMET", "DEFAULT")):
        """Get a config option, checking sections in order."""
        for section in sections:
            if self.parser.has_option(section, option):
                return ast.literal_eval(self.parser.get(section, option))
        return default

    def parse_config(self, section="DEFAULT"):
        self.project_name = self.parser.get("PROJECT", "project_name")
        super().parse_config(project_name=self.project_name, section="DEFAULT")

        self.countries = ast.literal_eval(self.parser.get("DEFAULT", "countries"))
        self.forecast_seasons = self._get_option(
            "forecast_seasons", [ar.utcnow().year]
        )
        self.plot_seasons = self._get_option(
            "plot_seasons", self.forecast_seasons
        )
        self.models = self._get_option("models", [])
        self.eo_plot = self._get_option(
            "eo_plot",
            ["ndvi", "cpc_tmax", "cpc_tmin", "chirps", "esi_4wk", "soil_moisture_as1", "soil_moisture_as2"],
        )
        self.eo_model = self._get_option(
            "eo_model",
            ["ndvi", "cpc_tmax", "cpc_tmin", "chirps", "esi_4wk", "soil_moisture_as1", "soil_moisture_as2"],
        )

        # Logos (optional)
        try:
            self.logo_harvest = self.dir_metadata / self.parser.get(
                "METADATA", "logo_harvest"
            )
            self.logo_geoglam = self.dir_metadata / self.parser.get(
                "METADATA", "logo_geoglam"
            )
        except Exception:
            self.logo_harvest = None
            self.logo_geoglam = None

    def read_statistics(self, country=None, read_countries=False, **kwargs):
        """Read zone/country info. country is optional when only reading countries."""
        if read_countries:
            path_countries = (
                self.dir_input / "statistics" / self.parser.get("DEFAULT", "zone_file")
            )
            self.df_countries = (
                pd.read_csv(path_countries)
                if path_countries.is_file()
                else pd.DataFrame()
            )
        if country is not None:
            super().read_statistics(country, read_countries=read_countries, **kwargs)

    def get_calendar_region_for_region(self, df, region):
        return df[df["region"] == region]["calendar_region"].values[0]

    def setup_country(self, country, scale, crop, growing_season):
        self.country = country
        self.scale = scale
        self.crop = crop
        self.growing_season = growing_season
        self.category = self.parser.get(country, "category")
        self.use_cropland_mask = self.parser.getboolean(country, "USE_CROPLAND_MASK")

        self.get_dirname(country)
        self.get_ccs_dataframe(country, scale, crop, growing_season)

        self.list_regions = self.df_ccs["region"].unique()
        self.list_calendar_regions = self.df_ccs["calendar_region"].unique()

        self.precip_var = (
            "chirps" if "chirps" in self.df_ccs.columns.values else "cpc_precip"
        )

        self.dir_output = self.dir_output / ar.now().format("MMMM_DD_YYYY")

    def setup_region(self, region, plot_season, type_region="region"):
        self.region = region
        self.calendar_region = self.get_calendar_region_for_region(self.df_ccs, region)
        self.plot_season = plot_season
        self.type_region = type_region

        if type_region == "region":
            self.df_region = self.df_ccs[self.df_ccs["region"] == region]
        elif type_region == "calendar_region":
            self.df_region = self.df_ccs[
                self.df_ccs["calendar_region"] == self.calendar_region
            ]
        elif type_region == "region_year":
            self.df_region = self.df_ccs[
                (self.df_ccs["region"] == region) & (self.df_ccs["year"] == plot_season)
            ]
        elif type_region == "calendar_region_year":
            self.df_region = self.df_ccs[
                (self.df_ccs["calendar_region"] == self.calendar_region)
                & (self.df_ccs["year"] == plot_season)
            ]
        else:
            raise ValueError(f"Unknown type_region: {type_region}")

        self.df_region.index = pd.to_datetime(self.df_region.index)

        folder = f"{self.crop}_s{self.growing_season}_{self.plot_season}"
        self.dir_agmet = (
            self.dir_output
            / "crop_condition"
            / self.category
            / self.country
            / folder
            / "condition"
        )

        (
            self.date_planting,
            self.date_greenup,
            self.date_senescence,
            self.date_harvesting,
        ) = self.get_calendar(self.region, self.plot_season)

    def create_run_combinations(self):
        all_combinations = []

        for country in self.countries:
            scales = ast.literal_eval(self.parser.get(country, "scales"))
            crops = ast.literal_eval(self.parser.get(country, "crops"))
            growing_seasons = ast.literal_eval(
                self.parser.get(country, "growing_seasons")
            )

            for scale in scales:
                for crop in crops:
                    for growing_season in growing_seasons:
                        all_combinations.extend(
                            list(
                                itertools.product(
                                    [country], [scale], [crop], [growing_season]
                                )
                            )
                        )

        return all_combinations

    def get_calendar(self, region, forecast_season):
        SEASON = "harvest_season"
        CAL = "crop_calendar"

        df_sub = self.df_ccs[self.df_ccs["region"] == region]
        df_sub.index = pd.to_datetime(df_sub.index)
        sr_cal = df_sub[df_sub[SEASON] == forecast_season][["doy", CAL]]

        sr_cal[CAL] = pd.to_numeric(sr_cal[CAL], errors="coerce")

        if sr_cal.empty:
            return np.NaN, np.NaN, np.NaN, np.NaN
        else:
            date_planting = (sr_cal[CAL] == 1).idxmax()
            date_greenup = (
                (sr_cal[CAL] == 2).idxmax() if len(sr_cal[sr_cal[CAL] == 2]) else None
            )
            date_senesc = (
                (sr_cal[CAL][::-1] == 2).idxmax()
                if len(sr_cal[sr_cal[CAL] == 3])
                else None
            )
            date_harvesting = (
                (sr_cal[CAL][::-1] == 3).idxmax()
                if len(sr_cal[sr_cal[CAL] == 3])
                else None
            )

            return date_planting, date_greenup, date_senesc, date_harvesting

    def get_ccs_dataframe(self, country, scale, crop, growing_season):
        dir_ccs = self.dir_output / self.dir_threshold / country

        self.df_ccs = pd.read_csv(
            dir_ccs / f"{country}_{crop}_s{growing_season}.csv", index_col=0
        )

        self.df_ccs["datetime"] = pd.to_datetime(self.df_ccs.index)
        self.df_ccs.index.name = None

    def get_closest_season(self, season):
        self.closest = nsmallest(
            5 + 1, range(2001, ar.utcnow().year), key=lambda x: abs(x - season)
        )

        if season in self.closest:
            self.closest.remove(season)

    def check_date(self, df, plot_season):
        last_valid_date = pd.to_datetime(df["chirps"].last_valid_index()).date()

        bool_year_check = ar.utcnow().year <= plot_season
        bool_date_check = (last_valid_date > self.date_planting.date()) & (
            last_valid_date < self.date_harvesting.date()
        )

        return bool_year_check, bool_date_check

    def add_precip_forecast(self, plot_season):
        bool_year_check, bool_date_check = self.check_date(self.df_ccs, plot_season)

        if bool_year_check & bool_date_check:
            base_dir = self.dir_output / self.dir_threshold / self.country / self.scale
            path_gefs = (
                base_dir / "cr" / "chirps_gefs"
                if self.use_cropland_mask
                else base_dir / self.crop / "chirps_gefs"
            )

            df_gefs = pd.read_csv(
                list(path_gefs.glob(self.df_region["region"].unique()[0] + "*.csv"))[0],
                header=None,
            )
            val_gefs = (
                float(df_gefs.values[1][5])
                if not math.isnan(float(df_gefs.values[1][5]))
                else 0.0
            )

            start_date = ar.utcnow().shift(days=+15).date()
            end_date = ar.utcnow().shift(days=+16).date()

            self.df_region.loc[start_date:end_date, "chirps_gefs"] = val_gefs
            self.df_ccs.loc[start_date.strftime("%Y-%m-%d"), "chirps_gefs"] = val_gefs


def create_title_for_plot(obj):
    """
    Args:
        obj:

    Returns:

    """
    region_name = obj.region.replace("_", " ").title()
    calendar_region_name = obj.calendar_region.replace("_", " ").title()
    country_name = obj.country.replace("_", " ").title()
    long_crop_name = utils.get_crop_name(obj.crop)

    # Get name of crop based on metadata/crop_per_season.csv file
    df_crop_per_season = pd.read_csv(obj.dir_metadata / "crop_per_season.csv")

    crop_name = df_crop_per_season[
        (df_crop_per_season["country"] == obj.country)
        & (df_crop_per_season["crop"] == obj.crop)
        & (df_crop_per_season["season"] == int(obj.growing_season))
    ]["name"].values

    crop_name = crop_name[0] if crop_name else long_crop_name.replace("_", " ").title()

    title_line_1 = f"{region_name} ({calendar_region_name}, {country_name})"
    title_line_2 = f"{crop_name} {obj.plot_season}"

    sup_title = f"{title_line_1}\n{title_line_2}"

    return sup_title


def loop_agmet(path_config_file=None):
    """
    Args:
    """
    # Create geo object
    obj = AgmetGeo(path_config_file)

    # Read in data on crop names in each country
    obj.read_statistics(read_countries=True)

    # Create combinations of run parameters
    all_combinations = obj.create_run_combinations()

    pbar = tqdm(all_combinations, total=len(all_combinations))
    for country, scale, crop, growing_season in pbar:
        # Setup country information
        obj.setup_country(country, scale, crop, growing_season)

        # Loop through seasons and plot for each admin_1 region and calendar region
        for plot_season in obj.plot_seasons:
            # Get list of the years that are closest to the year which we want to plot
            obj.get_closest_season(plot_season)

            for region in obj.list_regions:
                # Setup the region information
                obj.setup_region(region, plot_season, "region")

                # If available, add CHIRPS-GEFS data to the dataframe
                if obj.precip_var == "chirps":
                    obj.add_precip_forecast(plot_season)

                # Get crop calendar dates for region
                dates_calendar = [
                    obj.date_planting,
                    obj.date_greenup,
                    obj.date_senescence,
                    obj.date_harvesting,
                ]

                # TODO: Only plot if dates_calendar has valid dates
                # Create title for plot
                sup_title = create_title_for_plot(obj)

                ###############################################################
                # Agmet plots for regions
                ###############################################################
                plot.plots_ts_cur_yr(
                    obj.df_region,
                    obj.eo_plot,
                    closest=obj.closest,
                    dates_cal=dates_calendar,
                    frcast_yr=obj.plot_season,
                    logos=[obj.logo_harvest, obj.logo_geoglam],
                    dir_out=obj.dir_agmet / obj.scale,
                    sup_title=sup_title,
                    fname=f"{obj.region}.png",
                )

                ###############################################################
                # Agmet plots for calendar regions
                ###############################################################
                columns = obj.eo_model + ["month", "day", "yield"]
                if "chirps" in obj.df_region.columns:
                    bool_year_check, bool_date_check = obj.check_date(
                        obj.df_region, obj.plot_season
                    )

                    if bool_date_check and bool_year_check:
                        columns = obj.eo_model + [
                            "month",
                            "day",
                            "chirps_gefs",
                            "yield",
                        ]

                df_calendar_region = (
                    obj.df_region.groupby(
                        [
                            "country",
                            "calendar_region",
                            "harvest_season",
                            "doy",
                            "datetime",
                        ]
                    )[columns]
                    .mean()
                    .reset_index()
                )
                df_calendar_region.loc[:, "average_temperature"] = (
                    df_calendar_region["cpc_tmax"] + df_calendar_region["cpc_tmin"]
                ) / 2.0
                df_calendar_region.set_index(
                    pd.DatetimeIndex(df_calendar_region["datetime"]),
                    inplace=True,
                    drop=True,
                )
                df_calendar_region.index.name = None
                df_calendar_region.sort_values(by="datetime", inplace=True)

                # TODO: Only plot if dates_calendar has valid dates
                # Create title for plot
                sup_title = create_title_for_plot(obj)

                if not df_calendar_region.empty:
                    plot.plots_ts_cur_yr(
                        df_calendar_region,
                        obj.eo_plot,
                        closest=obj.closest,
                        dates_cal=dates_calendar,
                        frcast_yr=obj.plot_season,
                        logos=[obj.logo_harvest, obj.logo_geoglam],
                        dir_out=obj.dir_agmet / "district",
                        sup_title=sup_title,
                        fname=f"{obj.calendar_region}.png",
                    )


def run(path_config_files=[]):
    loop_agmet(path_config_files)


if __name__ == "__main__":
    run()
